import sys
import os
import tkinter as tk
from tkinter import ttk
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
import time

# Add the parent directory to the module search path
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from interface_board_pins import ADC_TERMINALS  # Import pin assignments


# CONFIG
ADC_PLOT_UPDATE_DELAY_MS = 50  # note: delay is additional to time it takes to update the chart
TAB_NOT_ACTIVE_CHECK_INTERVAL_MS = 2000
MAX_X_HISTORY_VALUES = 150
AUTO_SCALE_ENABLED = True
DEBUG = False


# Variables
updating_enabled = False
adc_channels = list(range(8))
data = {ch: [0] * MAX_X_HISTORY_VALUES for ch in adc_channels}  # Preallocate lists
time_data = list(range(-MAX_X_HISTORY_VALUES, 0))  # Simulated time axis


def create_adc_plot_tab(notebook, adc):
    frame = ttk.Frame(notebook)
    notebook.add(frame, text="ADC Plot")

    figure = Figure(figsize=(8, 5), dpi=100)
    ax = figure.add_subplot(1, 1, 1)

    ax.set_title("ADC Inputs")  # Set title
    ax.set_xlabel("Time (s)")  # X-axis label
    ax.set_ylabel("Voltage / Current (V / mA)")  # Y-axis label

    ax.set_xlim(-MAX_X_HISTORY_VALUES, 0)  # Keep time axis fixed
    canvas = FigureCanvasTkAgg(figure, master=frame)
    canvas.get_tk_widget().pack(fill=tk.BOTH, expand=True)

    # Initialize lines for fast updates
    lines = {}
    for terminal, config in ADC_TERMINALS.items():
        label = f"T{terminal} ({config['max_value']} {config['unit']})"  # Format: T1 (3.3V)
        lines[terminal] = ax.plot(time_data, data[terminal], label=label)[0]

    ax.legend()  # Show legend

    def update_plot():
        if not updating_enabled:
            frame.after(TAB_NOT_ACTIVE_CHECK_INTERVAL_MS, update_plot)
            return

        for terminal, config in ADC_TERMINALS.items():
            adc_channel = config["adc_channel"]
            max_value = config["max_value"]
            # Shift existing data left and
            # read new data + scale based on the correct max voltage
            data[terminal].pop(0)
            data[terminal].append(round(adc.read(adc_channel) * max_value / 4095, 2))

        # Recalculate limits (autoscale)
        if AUTO_SCALE_ENABLED:
            ax.set_ylim(bottom=0)  # Lower limit always 0
            ax.autoscale(enable=True, axis='y', tight=False)  # Enable autoscale for upper limit
            ax.relim()  # Recalculate limits
            ax.autoscale_view(scalex=False, scaley=True)  # Autoscale only y-axis
        else:
            ax.set_ylim(0, 24)  # Fixed range if autoscale is disabled

        for terminal in ADC_TERMINALS:
            lines[terminal].set_ydata(data[terminal])

        canvas.draw_idle()  # Efficient redraw
        frame.after(ADC_PLOT_UPDATE_DELAY_MS, update_plot)

    update_plot()


def set_updating_enabled(is_active):
    global updating_enabled
    updating_enabled = is_active
    if DEBUG: print(f"adc_plot tab: set updating_enabled to {updating_enabled}")
